// =======================================================================
//  File: AddEntity.aspx.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;

using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;

namespace Microsoft.Crm.Sandbox.Mobile
{
    /// <summary>
    /// This page allows the user to add an entity to the Mobile Client Configuration.
    /// </summary>
    public partial class AddEntity : System.Web.UI.Page
    {
		MetadataService _metadataService;
        ArrayList _entityList;    
        XmlDocument _xmlDoc;

        protected void Page_Load(object sender, EventArgs e)
        {            
            // Intialize the arraylist that will hold entities
            _entityList = new ArrayList();
            
            _metadataService = new MetadataService();
			_metadataService.Credentials = System.Net.CredentialCache.DefaultCredentials;
            
            // Load in the XML from the Draft version and make sure to not resolve namespaces
            _xmlDoc = new XmlDocument();
			_xmlDoc.XmlResolver = null; 
            _xmlDoc.Load(ConfigurationManager.AppSettings["DraftXmlLocation"]);
                       

            // Add all entities in the system to the array
            AddEntitiesToList();
            
            // Remove the entities already added
            FilterEntities();
            
            // Add the filtered entities to the dropdown on the page
            AddEntitiesToDropDown();
        }

        // Render out resources from resource file
        protected void RenderResources()
        {
            Title = Utils.GetResource("DialogTitle");

            AddEntityTitle.InnerText = Utils.GetResource("AddEntityTitle");
            AddEntityDescription.InnerText = Utils.GetResource("AddEntityDescription");

            OKButton.InnerText = Utils.GetResource("OKButtonText");
            CancelButton.InnerText = Utils.GetResource("CancelButtonText");
        }
        
        /// <summary>
        /// This adds all entities in the system (supported and custom)
        /// </summary>
        private void AddEntitiesToList()
        {
            // Add all supported system entities
            AddEntityToList(EntityName.account.ToString());
            AddEntityToList(EntityName.annotation.ToString());
            AddEntityToList(EntityName.customeraddress.ToString());
            AddEntityToList(EntityName.appointment.ToString());
            AddEntityToList(EntityName.incident.ToString());
            AddEntityToList(EntityName.contact.ToString());
            AddEntityToList(EntityName.competitor.ToString());
            AddEntityToList(EntityName.email.ToString());
            AddEntityToList(EntityName.fax.ToString());
            AddEntityToList(EntityName.invoice.ToString());
            AddEntityToList(EntityName.invoicedetail.ToString());
            AddEntityToList(EntityName.lead.ToString());
            AddEntityToList(EntityName.letter.ToString());
            AddEntityToList(EntityName.opportunity.ToString());
            AddEntityToList(EntityName.salesorder.ToString());
            AddEntityToList(EntityName.salesorderdetail.ToString());
            AddEntityToList(EntityName.phonecall.ToString());
            AddEntityToList(EntityName.quote.ToString());
            AddEntityToList(EntityName.quotedetail.ToString());
            AddEntityToList(EntityName.task.ToString());
            AddEntityToList(EntityName.systemuser.ToString());
            AddEntityToList(EntityName.campaign.ToString());
            AddEntityToList(EntityName.list.ToString());			        

            // Retrieve metadata for the server
            MetadataServiceSdk.Metadata metadata = _metadataService.RetrieveMetadata(MetadataFlags.EntitiesOnly);

            // Iterate through all entities and only add custom entities
            for (int i = 0; i < metadata.Entities.Length; i++)
            {
                if (metadata.Entities[i].IsCustomEntity &&
                    metadata.Entities[i].DisplayName != null &&
                    metadata.Entities[i].DisplayName.Length > 0)
                {
                    AddEntityToList(metadata.Entities[i].Name);
                }
            }

            // Sort the list by display name          
            _entityList.Sort(new EntityComparer());
        }

        /// <summary>
        /// Adds the specified entity to the list with the proper display name
        /// </summary>
        /// <param name="entityName">Schema name of entity being added</param> 
        private void AddEntityToList(string entityName)
        {
            // Retrieve entity
            EntityMetadata entity = _metadataService.RetrieveEntityMetadata(entityName, EntityFlags.EntityOnly);
            
            // Add entity to array with proper display name
            _entityList.Add(new ListItem(Utils.GetDisplayName(entity.Name), entity.Name));
        }

        /// <summary>
        /// Remove entities already selected
        /// </summary>
        private void FilterEntities()
        {
            // Create new array that will hold entities to delete
            ArrayList entitiesToRemove = new ArrayList();

            // Iterate through entity list
            for (int i = 0; i < _entityList.Count; i++)
            {
                ListItem item = _entityList[i] as ListItem;

                XmlNode selectedEntity = _xmlDoc.DocumentElement.SelectSingleNode(string.Format("Entities/Entity[@name='{0}']", item.Value));

                // If the entity was found in the config file add it to the list to delete
                if (selectedEntity != null)
                {
                    entitiesToRemove.Add(item);
                }
            }

            // Delete all entities already added
            for (int i = 0; i < entitiesToRemove.Count; i++)
            {
                _entityList.Remove(entitiesToRemove[i]);
            }            
        }

        /// <summary>
        /// Add entities to dropdown list
        /// </summary>
        private void AddEntitiesToDropDown()
        {
            // Add all entities to dropdown
            for (int i = 0; i < _entityList.Count; i++)
            {
                ListItem item = _entityList[i] as ListItem;
                entityList.Items.Add(item);
            }

            // If no entities were found make sure controls are disabled
            if (entityList.Items.Count == 0)
            {
                entityList.Enabled = false;
                OKButton.Disabled = true;
            }
        }
    }
}
