// =======================================================================
//  File: Field.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;

namespace Microsoft.Crm.Sandbox.Mobile
{
    /// <summary>
    /// Field struct used to store various information about an attribute
    /// </summary>
    public struct Field
    {
        private string _schemaName;
        private string _displayName;
        private string _labelName;
        private AttributeRequiredLevel _requiredLevel;

        /// <summary>
        /// Schema name of attribute
        /// </summary>
        public string SchemaName
        {
            get
            {
                return HttpUtility.HtmlEncode(this._schemaName);
            }
            set
            {
                this._schemaName = value;
            }
        }

        /// <summary>
        /// Display name of attribute
        /// </summary>
        public string DisplayName
        {
            get
            {
                return HttpUtility.HtmlEncode(this._displayName);
            }
            set
            {
                this._displayName = value;
            }
        }

        /// <summary>
        /// Label specified by user to use as display name in Mobile Client
        /// </summary>
        public string LabelName
        {
            get
            {
                return HttpUtility.HtmlEncode(this._labelName);
            }
            set
            {
                this._labelName = value;
            }
        }

        /// <summary>
        /// Required Level of attribute
        /// </summary>
        public AttributeRequiredLevel RequiredLevel
        {
            get
            {
                return this._requiredLevel;
            }
            set
            {
                this._requiredLevel = value;
            }
        }
    }
}