// =======================================================================
//  File: User.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Caching;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;

//namespace Microsoft.Crm.Sandbox.Mobile
//{
/// <summary>
/// This user object holds various information about the user
/// </summary>
public class User
{
	private RolePrivilege[] _userPrivileges;
	private Guid _userId;
	private Guid _organizationId;

	public User() {}
	
	/// <summary>
	/// Systemuser ID of the user
	/// </summary>
	public Guid UserId
	{
		get
		{
			return _userId;
		}
		set
		{
			_userId = value;
		}	
	}

	/// <summary>
	/// Organization ID of the organization the user belongs to
	/// </summary>
	public Guid OrganizationId
	{
		get
		{
			return _organizationId;
		}
		set
		{
			_organizationId = value;
		}
	}

	/// <summary>
	/// All the privileges the current user has
	/// </summary>
	public RolePrivilege[] UserPrivileges
	{
		get
		{
			return _userPrivileges;
		}
		set
		{
			_userPrivileges = value;
		}
	}		

	/// <summary>
	/// Determines if the current user has the privilege requested
	/// </summary>
	/// <param name="privilegeId">Privilege ID to check for</param>
	/// <returns>If the privilege was found </returns>
	public bool GetPrivilege(Guid privilegeId)
	{
		// Check to see if the current user the privilege requested
		for (int i = 0; i < this.UserPrivileges.Length; i++)
		{		
			if (this.UserPrivileges[i].PrivilegeId == privilegeId)
			{
				return true;
			}
		}

		return false;
	}
}
//}