// =======================================================================
//  File: Utils.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Collections;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;

namespace Microsoft.Crm.Sandbox.Mobile
{
    /// <summary>
    /// This class contains helper functions used in various pages
    /// </summary>
    public class Utils
    {
        public Utils(){}
        
        /// <summary>
        /// Retrieve the object type code for the specified entity
        /// </summary>
        /// <param name="entityName">Schema name of the entity</param>
        /// <returns>Object type code of entity</returns>
        public static int GetObjectTypeCode(string entityName)
        {
            // Create MetadataService
            MetadataService metadataService = new MetadataService();
            metadataService.Credentials = System.Net.CredentialCache.DefaultCredentials;

            // Retrieve entity
            EntityMetadata entity = metadataService.RetrieveEntityMetadata(entityName, EntityFlags.EntityOnly);

            return entity.ObjectTypeCode;              
        }

        /// <summary>
        /// Retrieve the display name for the specified entity
        /// </summary>
        /// <param name="entityName">Schema name of the entity</param>
        /// <returns>Display name of the entity</returns>
        public static string GetDisplayName(string entityName)
        {
            // Create MetadataService
            MetadataService metadataService = new MetadataService();
            metadataService.Credentials = System.Net.CredentialCache.DefaultCredentials;

            // Retrieve entity
            EntityMetadata entity = metadataService.RetrieveEntityMetadata(entityName, EntityFlags.EntityOnly);

            return HttpUtility.HtmlEncode(entity.DisplayName);
        }

        /// <summary>
        /// Checks to see if a bit is in a value
        /// </summary>
        /// <param name="val">Value being checked</param>
        /// <param name="bit">Bit being looked for</param>
        /// <returns>Return result of check</returns>
        public static bool CheckBit(uint val, uint bit)
        {
            return ((bit & val) == bit);
        }

		// Retrieve resource from global resource file
		public static string GetResource(string key)
		{
			return HttpContext.GetGlobalResourceObject("CrmMobileClient", key).ToString();
		}

    }   

    /// <summary>
    /// Used by arraylists to sort the field structs by display name
    /// </summary>
    public class FieldComparer : IComparer
    {
        public FieldComparer() { }

        public int Compare(object obj1, object obj2)
        {
            Field field1 = (Field)obj1;
            Field field2 = (Field)obj2;

            return field1.DisplayName.CompareTo(field2.DisplayName);
        }
    }

    /// <summary>
    /// Used by arraylists to sort list items by their text
    /// </summary>
    public class EntityComparer : IComparer
    {
        public EntityComparer() { }

        public int Compare(object obj1, object obj2)
        {
            ListItem list1 = obj1 as ListItem;
            ListItem list2 = obj2 as ListItem;

            return list1.Text.CompareTo(list2.Text);
        }
    } 
    
    
}
