﻿// =======================================================================
//  File: EntityConfig.js
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

// These two variables are the constants for required fields
var SYSTEM_REQUIRED = "1";
var BUSINESS_REQUIRED = "2";
var _bSaving = false;

// Removes a field from the selected fields
function MoveLeft()
{
    if (!IsNull(_oRow))
    {
        // You can only remove fields from the selected fields list
        if (GetListName() == "rtnObjList")
        {     
            // Do not allow user to remove required field
            if (_oRow.req != SYSTEM_REQUIRED && _oRow.req != BUSINESS_REQUIRED)
            {     
                // Create new row in opposite list
                CreateNewRow(_oRow.name, _oRow.innerText, _oRow.label, _oRow.req, objList);

                // Remove row from current list
                rtnObjList.firstChild.deleteRow(_oRow.rowIndex);            
                
                // "Deselect" current row
                _oRow = null;
                
                currentLabel.innerText = "";
    		    
    		    // Sort available fields list to be in alphabetical order
		        SortList(objList);
		    }
		    else
		    {
		        alert(REMOVE_REQUIRED_FIELD_WARNING);
		    }
        }
    }      
}

// Add a field to the selected fields list
function MoveRight()
{
    if (!IsNull(_oRow))
    {
        // You can only add fields from the available fields list
        if (GetListName() == "objList")
        {
            CreateNewRow(_oRow.name, _oRow.innerText, _oRow.label, _oRow.req, rtnObjList);

            objList.firstChild.deleteRow(_oRow.rowIndex); 
            
            _oRow = null;  
            
            currentLabel.innerText = "";      
        }
    }
}   

// Fires when user double-clicks a field
function MoveToOtherList()
{
    if (GetListName() == "objList")
    {
        MoveRight();
    }
    else
    {
        MoveLeft();
    }
}   

// Custom object to hold information about attributes
function Attribute(sName, sDisplayName, sLabel, sRequiredLevel)
{
	this.Name = sName;
	this.DisplayName = sDisplayName;
	this.Label = sLabel;
	this.RequiredLevel = sRequiredLevel;
}   

// Compares the display names of two attributes
function CompareDisplayName(attribute1, attribute2) 
{  
    if (attribute1.DisplayName < attribute2.DisplayName) 
    { 
        return -1; 
    }
    else 
    {
        if (attribute1.DisplayName > attribute2.DisplayName) 
        { 
            return 1;
        }
        else 
        { 
            return 0; 
        }
    }
}  

// Sorts the specified list
// NOTE: Currently doesn't allow sorting on selected fields list (just need to deselect row first)
function SortList(oList)
{   
    var oTable = oList.firstChild;
    var oRows = oTable.rows;
    var iRowCount = oTable.rows.length;
    var aAvailableFields = new Array(iRowCount);
    
    // For all the rows in the list, create new attribute objects and remove the row from the table
    for (var i = 0; i < iRowCount; i++)
    {
        aAvailableFields[i] = new Attribute(oRows[0].name, oRows[0].innerText, oRows[0].label, oRows[0].req); 
        oTable.deleteRow(0);
    }
      
    // Sort by display name
    aAvailableFields.sort(CompareDisplayName);
    
    // Recreate table
    for (var i = 0; i < iRowCount; i++)
    {
        CreateNewRow(aAvailableFields[i].Name, aAvailableFields[i].DisplayName, aAvailableFields[i].Label, aAvailableFields[i].RequiredLevel, oList);
    }
}

// Renders a new row for a field
function CreateNewRow(sName, sDisplay, sLabel, sReq, oDestList)
{
    var oTable	    = oDestList.firstChild;
    var oRow	    = oTable.insertRow(oTable.rows.length);
    var oCell	    = oRow.insertCell();

    oCell.className	= "sel";
    oCell.noWrap	= true;
    oCell.innerText	= sDisplay;   

    oRow.name       = sName;
    oRow.selected   = 0;
    oRow.label      = sLabel;
    oRow.req        = sReq;

    oRow.attachEvent("onclick", selectRow);
    oRow.attachEvent("ondblclick", MoveToOtherList);
    oRow.attachEvent("onkeydown", HandleKeyDown);
}

// Builds up submit XML for EntityConfigPage
function BuildFormXML(bEncode)
{
    var sXml = "<Entity name=\"";
    sXml += crmForm.ConfigEntityName.value;
    sXml += "\" readOnly=\"";
    sXml += crmForm.radReadOnly.checked ? "1" : "0"; 
    sXml += "\">";
    
    sXml += "<Fields>"   
    
    var oTable = rtnObjList.firstChild;
    var oRowCount = oTable.rows.length;
    
    for (var i = 0; i < oRowCount; i++)
    {
        sXml += "<Field name=\"" + oTable.rows[i].name + "\" label=\"";
        
        if (bEncode)
        {
            sXml += XmlEncode(oTable.rows[i].label);
        }
        else
        {
            sXml += oTable.rows[i].label;
        }          
         
        sXml += "\" />";
    }   
    
    sXml += "</Fields></Entity>"; 
    
    return sXml;    
}

// Opens the dialog to edit a field's label
function EditLabel()
{
	if (!IsNull(_oRow))
	{
		var sLabel = window.showModalDialog("/MobileClient/EditLabel.aspx?sCurrentLabel=" + _oRow.label, window, "dialogHeight: 200px; dialogWidth: 400px;help:0;status:0;scroll:0");	

		// Set the new label if there is one
		if (!IsNull(sLabel))
		{
			_oRow.label = sLabel;
			currentLabel.innerText = sLabel;
		}
    }
}

// Prompt the user if there are changes that have not been saved
function Close()
{     
    var sFormXml = BuildFormXML(false);
    
    if (sFormXml != crmForm.CurrentXml.value && !_bSaving)
    {
        event.returnValue = "\n--------------------------------------------------------------------------\n\n" + CHANGES_WITHOUT_SAVE_WARNING + "\n\n--------------------------------------------------------------------------\n";
    }
}

// Handles key down presses on selected row
function HandleKeyDown()
{      
    var iKeyPressed = event.keyCode;
    
    // Make sure a row is selected (every method should check anyway)
    if (!IsNull(_oRow))
    {
		switch (iKeyPressed)
		{
			case ENTER_KEY:
				EditLabel();
				break;
			case LEFT_KEY:
				MoveLeft();
				break;
			case UP_KEY:
			
				if (_oRow.rowIndex != 0)
				{                
					if (!IsNull(_oRow))
					{
						_oRow.firstChild.className = "sel";
						_oRow.selected = 0;
					}
	                
					var oRow = _oRow.previousSibling;    
	        	    
					oRow.firstChild.className = "currSel";
					oRow.selected = 1;
	        		
					_oRow = oRow;	
	                
					_oRow.scrollIntoView(true);
	        	    
					currentLabel.innerText = _oRow.label;
				}	
				
				break;    
			case RIGHT_KEY:
				MoveRight();
				break;
			case DOWN_KEY:
				var oList  = document.getElementById(GetListName());
				var oTable = oList.firstChild;
	            
				if (_oRow.rowIndex < oTable.rows.length - 1)
				{             
					if (!IsNull(_oRow))
					{
						_oRow.firstChild.className = "sel";
						_oRow.selected = 0;
					}
	                
					var oRow = _oRow.nextSibling;
	        	    
					oRow.firstChild.className = "currSel";
					oRow.selected = 1;
	        		
					_oRow = oRow;
	                
					_oRow.scrollIntoView(false);	
	        	    
					currentLabel.innerText = _oRow.label;  	       
				}   
				
				break;
			default:
				break;
		}           
	}  
}


