﻿// =======================================================================
//  File: Global.js
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

// This is the currently selected row
var _oRow = null;

// Determines if we are on the Entity Config Page
var _bEntityConfig = false;

var ENTER_KEY = 13;
var DELETE_KEY = 46;

var LEFT_KEY = 37;
var UP_KEY = 38;
var RIGHT_KEY = 39;
var DOWN_KEY = 40;

// This method sets the currently selected row as the current row
function selectRow()
{       
	var oRow = event.srcElement;

	while (oRow.tagName != "TR")
	{
		oRow = oRow.parentElement;
	}
	
	// If the user selects the same row, do nothing
	if (oRow == _oRow)
	{
	    return;
	}
	else
	{
	    // The HomePage and EntityConfigPage rows are slightly different
	    if (_bEntityConfig)
	    {
	        // If there is already a row selected, "deselect" it
	        if (!IsNull(_oRow))
	        {
	            _oRow.firstChild.className = "sel";
	            _oRow.selected = 0;
	        }
    	    
		    oRow.firstChild.className = "currSel";
		    oRow.selected = 1;
    		
		    _oRow = oRow;	
		    
		    currentLabel.innerText = _oRow.label;
		}
		else
		{
		    if (!IsNull(_oRow))
	        {
	            _oRow.cells[0].className = "grid sel";
	            _oRow.cells[1].className = "grid sel"; 
	            _oRow.selected = 0;
	        }
	        
	        document.getElementById("SelectedEntityName").value = oRow.name;
	        
	        oRow.cells[0].className = "grid currSel";
	        oRow.cells[1].className = "grid currSel"; 
	       
		    oRow.selected = 1;
    		
		    _oRow = oRow;		    
		}
	}
}

// Retrieve current row's list name
function GetListName()
{
    return _oRow.parentElement.parentElement.parentElement.id;
}

// Moves a row up
function MoveUp()
{
    var oTable	= rtnObjList.firstChild;
    
    if (!IsNull(_oRow))
    {
        // You can only move rows that are on the selected list
        if (GetListName() == "rtnObjList")
        {
            if (_oRow.rowIndex != 0)
            {
                oTable.moveRow( _oRow.rowIndex, _oRow.rowIndex - 1 );                
                
                // We want to save the config if we are on the HomePage
                if (!_bEntityConfig)
                {
                    Save();
                }   
            }     
        } 
    }   
}

// Moves a row down
function MoveDown()
{
    var oTable	= rtnObjList.firstChild;
    
    if (!IsNull(_oRow))
    {
        if (GetListName() == "rtnObjList")
        {       
            if (_oRow.rowIndex < oTable.rows.length - 1)
            {     
                oTable.moveRow( _oRow.rowIndex, _oRow.rowIndex + 1 );   
                
                if (!_bEntityConfig)
                {
                    Save();
                }               
            }       
        } 
    }      
}

// NOTE: Taken from Microsoft CRM Code Base

function IsNull(o)
{
    return ("undefined" == typeof(o) || "unknown" == typeof(o) || null == o);
}

function formatString()
{
    if (arguments.length == 0)
    {
        return "";
    }
   
    var s = arguments[0];
    var aArguments = arguments;
   
    if (arguments.length > 1 && isArray(arguments[1]))
    {
        aArguments = arguments[1];

        aArguments.unshift(s);
    }
    
    for (var i = 1; i < aArguments.length; i++)
    {
        var argIndex = i - 1;
        s = s.replace(new RegExp("\\{" + argIndex + "\\}", "g"), aArguments[i].toString());
    }
    
    return s;
}

function isArray(item)
{
    return item != null && ((item instanceof Array) || (typeof item.splice) == "function");
}

// Modified from Microsoft CRM Code Base 
function XmlEncode(s)
{
	if (IsNull(s))
	{
		return "";
	}
	if (typeof(s) != "string")
	{
		s = s.toString();
	}

	s = escape(s);
	s = s.replace(/%20/g, " ");
	s = s.replace(/%u(\w{4})/g,	"&#x$1;");
	s = s.replace(/%(\w{2})/g,	"&#x$1;");
	return s;
}

// Modified from Microsoft CRM Code Base 
function ResetGlow(o)
{
    if (!o)
    {
        return false;
    }
   
    with (o.runtimeStyle)
    {
        padding = formatString("3px {0}px 3px {1}px", o.pr, o.pl);
        borderStyle = "none";
        backgroundColor = "";    
    }
}

// Modified from Microsoft CRM Code Base 
function SetGlow(o)
{
    if (!o)
    {
        return false;
    }

    with (o.runtimeStyle)
    {
        padding = formatString("2px {0}px 2px {1}px", o.pr - 1, o.pl - 1);
        border = "1px solid #485673";
        backgroundColor = "#edf1d5";
    }
}