// =======================================================================
//  File: MetadataCache.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Caching;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;

/// <summary>
/// This metadata cache is used to cache entity metadata that is commonly used throughout the application
/// </summary>
public class MetadataCache
{
	// Unique Key used to inserts objects into the cache
	private const string CACHE_KEY = "MicrosoftCrmMobileMetadataKey";

	private MetadataCache() { }

	static MetadataCache() { }

	/// <summary>
	/// Retrieves an entity that is cached
	/// </summary>
	/// <param name="entityName">Schema name of the entity</param>
	/// <returns>EntityMetadata of the entity</returns>
	public static EntityMetadata GetEntityMetadata(string entityName)
	{
        //todo: remove?
		entityName = entityName.ToLower();

		string metadataKey = CACHE_KEY + entityName;

		// Retrieve entity metadata from cache
		EntityMetadata entity = HttpContext.Current.Cache[metadataKey] as EntityMetadata;

		// If the entity metadata was found, return it
		if (entity != null)
		{
			return entity;
		}

		// Create CRM Service with proper credentials
		MetadataService metadataService = new MetadataService();
		metadataService.Credentials = System.Net.CredentialCache.DefaultCredentials;

		// Retrieve entity metadata with all flags			
		entity = metadataService.RetrieveEntityMetadata(entityName, EntityFlags.All);

		// Insert the entity metadata into the cache
		HttpContext.Current.Cache.Insert(metadataKey, entity, null, Cache.NoAbsoluteExpiration, TimeSpan.FromMinutes(15));

		return entity;
	}

	/// <summary>
	/// Retrieves the specified attribute from the cached entity
	/// </summary>
	/// <param name="entityName">Schema name of the entity</param>
	/// <param name="propertyName">Attribute schema name</param>
	/// <returns>The attributemetadata of the attribute</returns>
	public static AttributeMetadata GetAttributeMetadata(string entityName, string propertyName)
	{
		//todo: Remove?
		entityName = entityName.ToLower();
		propertyName = propertyName.ToLower();

		// Get the entity from the cache
		EntityMetadata entity = GetEntityMetadata(entityName);

		// Retrieve the attribute from the entity
		for (int i = 0; i < entity.Attributes.Length; i++)
		{
			if (propertyName == entity.Attributes[i].Name)
			{
				return entity.Attributes[i];
			}
		}

		return null;
	}
}
