// =======================================================================
//  File: Mobile.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Security.Principal;
using System.Runtime.InteropServices;
using System.Net;
using System.Xml;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;

/// <summary>
/// Summary description for Mobile
/// </summary>
public class Mobile : System.Web.UI.Page
{
    protected CrmService _crmService;
    protected MetadataService _metaService;
	private XmlDocument _xmlDoc;

	public const string MASTER_PAGE_CONTENT = "$PageContent$";
	
	/// <summary>
	/// This represents the base class that all pages in this application
	/// should inherit. It gives access to the crmservice, metadata service,
	/// and the xml config file. Many of the values that the methods in this
	/// class use are stored in the web.config
	/// </summary>
    public Mobile(){}

	//Crm Service Accessor
	public CrmService CurrentCrmService
	{
		get
		{
			if (_crmService == null)
			{				
				_crmService = new CrmService();
				_crmService.Credentials = CredentialCache.DefaultCredentials;				
				
				return _crmService;
			}
			else
			{
				return _crmService;
			}
		}
		set
		{
			_crmService = value;
		}
	}
	
	//Metadata Service Accessor
	public MetadataService CurrentMetaService
	{
		get
		{
			if (_metaService == null)
			{
				_metaService = new MetadataService();
				_metaService.Credentials = CredentialCache.DefaultCredentials;				
				
				return _metaService;
			}
			else
			{
				return _metaService;
			}
		}
		set
		{
			_metaService = value;
		}
	}

	//Config Xml Accessor
	public XmlDocument MobileConfig
	{
		get
		{
			if(_xmlDoc == null)
			{
				_xmlDoc = new XmlDocument();
                _xmlDoc.XmlResolver = null;				
				_xmlDoc.Load(ConfigurationManager.AppSettings["MobileConfigPath"]);
				
				return _xmlDoc;
			}
			else
			{
				return _xmlDoc;
			}
		}
		set
		{
			_xmlDoc = value;
		}
	}
	
	//Method returns the default value of the page title, it's specified in web.config
	public string GetDefaultPageTitle()
	{
		return ConfigurationManager.AppSettings["PageTitle"];		
	}

	//Method returns the maximum instances that can be returned in a query, specified in web.config	
	public int GetMaxInstancesReturned()
	{
		return Int32.Parse(ConfigurationManager.AppSettings["MaxInstancesReturned"]);
	}
}
