// =======================================================================
//  File: OrganizationCache.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Caching;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;

/// <summary>
/// This organization cache is used to cache CRM Organization information so it can be easily retrieve from
///  other parts of the application.
/// </summary>
public class OrganizationCache
{
	// Unique Key used to inserts objects into the cache
	private const string CACHE_KEY = "MicrosoftCrmMobileOrganizationKey";

	private OrganizationCache() { }

	static OrganizationCache() { }

	/// <summary>
	/// Retrieves current Organization from cache or by calling the CRM SDK
	/// </summary>
	/// <param name="user">Current user so we can retrieve the appropriate Organization</param>
	/// <returns>Current organization as Organiaztion (not organization - CRM TYPE)</returns>
	public static Organization GetOrganization(User user)
	{
		string orgKey = CACHE_KEY + user.OrganizationId.ToString("b");

		// Retrieve organization from cache
		Organization org = HttpContext.Current.Cache[orgKey] as Organization;

		// If the organization was found, return it
		if (org != null)
		{
			return org;
		}

		// Create CRM Service with proper credentials
		CrmService crmService = new CrmService();
		crmService.Credentials = System.Net.CredentialCache.DefaultCredentials;

		// Retrieve Organization with all attributes
		organization userOrganization = crmService.Retrieve(EntityName.organization.ToString(), user.OrganizationId, new AllColumns()) as organization;

		// Set settings on Organization
		org = new Organization(userOrganization);

		// Insert the organization into the cache
		HttpContext.Current.Cache.Insert(orgKey, org, null, Cache.NoAbsoluteExpiration, TimeSpan.FromMinutes(15));

		return org;
	}
}
