// =======================================================================
//  File: conditionexpressionhelpercollection.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================


using System;
using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.Helpers;

namespace Microsoft.Crm.Sandbox.Mobile.Helpers.Collection
{
	public class ConditionExpressionHelperCollection : System.Collections.CollectionBase
	{
		#region Properties

		/// <summary>
		/// Provides access to the underlying collection of ConditionExpressionHelpers
		/// </summary>
		public ConditionExpressionHelper this[int index]
		{
			get
			{
				return (ConditionExpressionHelper)List[index];
			}
		}

		#endregion

		#region Methods

		/// <summary>
		/// Used to add a condition to the collection
		/// </summary>
		/// <param name="attributeName">Specifies which attribute the condition will be applied to</param>
		/// <param name="conditionOperator">Specifies which condition will be applied</param>
		public void AddCondition(string attributeName, ConditionOperator conditionOperator)
		{
			ConditionExpressionHelper condition = new ConditionExpressionHelper(attributeName, conditionOperator);
			List.Add(condition);
		}

		/// <summary>
		/// Used to add a condition to the collection
		/// </summary>
		/// <param name="attributeName">Specifies which attribute the condition will be applied to</param>
		/// <param name="conditionOperator">Specifies which condition will be applied</param>
		/// <param name="value">Specifies what parameter the condition will use to apply to the attribute specified</param>
		public void AddCondition(string attributeName, ConditionOperator conditionOperator, object value)
		{
			ConditionExpressionHelper condition = new ConditionExpressionHelper(attributeName, conditionOperator, value);
			List.Add(condition);
		}

		/// <summary>
		/// Used to add a condition to the collection
		/// </summary>
		/// <param name="attributeName">Specifies which attribute the condition will be applied to</param>
		/// <param name="conditionOperator">Specifies which condition will be applied</param>
		/// <param name="value">Specifies what parameters the condition will use to apply to the attribute specified</param>
		public void AddCondition(string attributeName, ConditionOperator conditionOperator, object[] values)
		{
			ConditionExpressionHelper condition = new ConditionExpressionHelper(attributeName, conditionOperator, values);
			List.Add(condition);
		}

		/// <summary>
		/// Used to add a condition to the collection
		/// </summary>
		/// <param name="condition">A ConditionExpressionHelper object that will be added to the collection</param>
		public void AddCondition(ConditionExpressionHelper condition)
		{
			List.Add(condition);
		}

		internal ConditionExpression[] ToConditionExpressionArray()
		{
			// initialize the condition expression array
			ConditionExpression[] returnArray = new ConditionExpression[List.Count];

			// populate the array
			for (int index = 0; index < returnArray.Length; index++)
			{
				returnArray[index] = (ConditionExpression)((ConditionExpressionHelper)List[index])._conditionExpression;
			}

			return returnArray;
		}

		#endregion
	}
}
