// =======================================================================
//  File: orderexpressioncollection.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================


using System;
using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;

namespace Microsoft.Crm.Sandbox.Mobile.Helpers.Collection
{
	public class OrderExpressionCollection : System.Collections.CollectionBase
	{
		#region Properties

		/// <summary>
		/// Provides access to the underlying collection of OrderExpressions
		/// </summary>
		public OrderExpression this[int index]
		{
			get
			{
				return (OrderExpression)List[index];
			}
		}

		#endregion

		#region Methods

		/// <summary>
		/// Adds an order to the collection
		/// </summary>
		/// <param name="attributeName">The name of the attribute where an order will be applied for the query</param>
		/// <param name="orderType">An OrderType enum that specifies what kind of ordering to apply on the attribute</param>
		public void Add(string attributeName, OrderType orderType)
		{
			OrderExpression order = new OrderExpression();

			order.AttributeName	= attributeName;
			order.OrderType		= orderType;

			List.Add(order);
		}

		internal OrderExpression[] ToArray()
		{
			// initialize the string array to the number of orders in the internal list 
			OrderExpression[] returnArray = new OrderExpression[List.Count];

			// populate the array
			for (int index = 0; index < returnArray.Length; index++)
			{
				returnArray[index] = (OrderExpression)List[index];
			}

			return returnArray;
		}

		#endregion
	}
}
