// =======================================================================
//  File: BreadCrumbsControl.ascx.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

//namespace Microsoft.Crm.Sandbox.Mobile
//{
    /// <summary>
    /// This control allows the user to navigate through the sitemap of the Mobile Application
    /// </summary>
    public partial class BreadCrumbsControl : System.Web.UI.UserControl
    {
        private string _homePageUrl;
        private string _homePageName;
        
        private string _entityHomePageUrl;
        private string _entityHomePageName;
        
        private string _recordPageUrl;
        private string _recordPageName;
        
        private string _relatedEntitiesUrl;
        private string _relatedEntitiesName;
        
        private string _relatedEntityPageUrl;
        private string _relatedEntityPageName;

        /// <summary>
        /// URL of the HomePage
        /// </summary>
        public string HomePageUrl
        {
            get
            {
                return _homePageUrl;
            }
            set
            {
                _homePageUrl = value;
            }
        }

        /// <summary>
        /// Text displayed for the HomePage link
        /// </summary>
        public string HomePageName
        {
            get
            {
                return _homePageName;
            }
            set
            {
                _homePageName = value;
            }
        }

        /// <summary>
        /// URL of the HomePage of the current entity
        /// </summary>
        public string EntityHomePageUrl
        {
            get
            {
                return _entityHomePageUrl;
            }
            set
            {
                _entityHomePageUrl = value;
            }
        }

        /// <summary>
        /// DisplayCollectioName of the current entity
        /// </summary>
        public string EntityHomePageName
        {
            get
            {
                return _entityHomePageName;
            }
            set
            {
                _entityHomePageName = value;
            }
        }

        /// <summary>
        /// URL of the current record
        /// </summary>
        public string RecordPageUrl
        {
            get
            {
                return _recordPageUrl;
            }
            set
            {
                _recordPageUrl = value;
            }
        }

        /// <summary>
        /// Name of the current record
        /// </summary>
        public string RecordPageName
        {
            get
            {
                return _recordPageName;
            }
            set
            {
                _recordPageName = value;
            }
        }

		/// <summary>
		/// URL of the HomePage of the related entity
		/// </summary>
		public string RelatedEntitiesUrl
		{
			get
			{
				return _relatedEntitiesUrl;
			}
			set
			{
				_relatedEntitiesUrl = value;
			}
		}

		/// <summary>
		/// DisplayCollectioName of the related entity
		/// </summary>
		public string RelatedEntitiesName
		{
			get
			{
				return _relatedEntitiesName;
			}
			set
			{
				_relatedEntitiesName = value;
			}
		}

		/// <summary>
		/// URL of the related record
		/// </summary>
		public string RelatedEntityPageUrl
		{
			get
			{
				return _relatedEntityPageUrl;
			}
			set
			{
				_relatedEntityPageUrl = value;
			}
		}

		/// <summary>
		/// Name of the related record
		/// </summary>
		public string RelatedEntityPageName
		{
			get
			{
				return _relatedEntityPageName;
			}
			set
			{
				_relatedEntityPageName = value;
			}
		}

        protected override void Render(HtmlTextWriter writer)
        {
            writer.Write("<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" class=\"breadCrumbs\"><tr><td class=\"breadCrumbs\">");
            
            if ((HomePageUrl != null && HomePageName != null) && (HomePageUrl.Length != 0 && HomePageName.Length != 0))
            {
                writer.Write("<a href=\"{0}\" id=\"BCHomePage\" class=\"breadCrumbs\">{1}</a>", HomePageUrl, HomePageName);

                if ((EntityHomePageUrl != null && EntityHomePageName != null) && (EntityHomePageUrl.Length != 0 && EntityHomePageName.Length != 0))
                {
                    writer.Write(' ');
                    writer.Write(MobileUtility.GetResource("BreadCrumbsSeparator"));
                    writer.Write(' ');
                    
                    writer.Write("<a href=\"{0}\" id=\"BCEntityHomePage\" class=\"breadCrumbs\">{1}</a>", EntityHomePageUrl, EntityHomePageName);

                    if ((RecordPageUrl != null && RecordPageName != null) && (RecordPageUrl.Length != 0 && RecordPageName.Length != 0)) 
                    {
						writer.Write(' ');
						writer.Write(MobileUtility.GetResource("BreadCrumbsSeparator"));
						writer.Write(' ');
						
                        writer.Write("<a href=\"{0}\" id=\"BCRecordPage\" class=\"breadCrumbs\">{1}</a>", RecordPageUrl, RecordPageName);

						if ((RelatedEntitiesUrl != null && RelatedEntitiesName != null) && (RelatedEntitiesUrl.Length != 0 && RelatedEntitiesName.Length != 0))
						{
							writer.Write(' ');
							writer.Write(MobileUtility.GetResource("BreadCrumbsSeparator"));
							writer.Write(' ');
							
							writer.Write("<a href=\"{0}\" id=\"BCRecordPage\" class=\"breadCrumbs\">{1}</a>", RelatedEntitiesUrl, RelatedEntitiesName);

							if ((RelatedEntityPageUrl != null && RelatedEntityPageName != null) && (RelatedEntityPageUrl.Length != 0 && RelatedEntityPageName.Length != 0))
							{
								writer.Write(' ');
								writer.Write(MobileUtility.GetResource("BreadCrumbsSeparator"));
								writer.Write(' ');
								
								writer.Write("<a href=\"{0}\" id=\"BCRecordPage\" class=\"breadCrumbs\">{1}</a>", RelatedEntityPageUrl, RelatedEntityPageName);
							}
						}
                    }
                }
            }

            writer.Write("</td></tr></table>");
        }
    }
//}