// =======================================================================
//  File: EntityEdit.aspx.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Collections;
using System.Configuration;
using System.Drawing;
using System.Globalization;
using System.Threading;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;

/// <summary>
/// This page is responsible for edit form functionality of entity instance.
/// </summary>
public partial class EntityEdit : Mobile
{
	protected EntityForm _entityForm;

	protected void Page_Load(object sender, EventArgs e)
	{

		//Create the entity form
		_entityForm = new EntityForm(this, false);

		//Add the entity form
		TheForm.Controls.Add(_entityForm);

		BreadCrumbsControl breadCrumbs = Master.FindControl("BreadCrumbs") as BreadCrumbsControl;

		breadCrumbs.HomePageName = MobileUtility.GetResource("HomePageText");
		breadCrumbs.HomePageUrl = "Home.aspx";

		string referencingEntity = HttpUtility.HtmlEncode(Request.QueryString["referencingentity"]);
		string referencingRecordName = HttpUtility.HtmlEncode(Request.QueryString["referencingrecordname"]);
		string referencingId = HttpUtility.HtmlEncode(Request.QueryString["referencingid"]);		

		// This code is similiar to the EntityReadOnly page except the else allows null values becaue the edit page
		// also acts as the new page. We may convert this to a method in the future to handle both cases.
		if ((referencingEntity != null && referencingEntity.Length != 0) &&
			(referencingRecordName != null && referencingRecordName.Length != 0) &&
			(referencingId != null && referencingId.Length != 0))
		{
			Guid referencingGuid = new Guid(referencingId);
			
			EntityMetadata referencingMetadata = MetadataCache.GetEntityMetadata(referencingEntity);
			breadCrumbs.EntityHomePageName = HttpUtility.HtmlEncode(referencingMetadata.DisplayCollectionName);
			breadCrumbs.EntityHomePageUrl = "EntityHome.aspx?entity=" + referencingMetadata.Name;

			breadCrumbs.RecordPageName = referencingRecordName;
			breadCrumbs.RecordPageUrl = "EntityReadOnly.aspx?entity=" + referencingMetadata.Name + "&" + referencingMetadata.PrimaryKey + "=" + referencingGuid.ToString("b");
			
			breadCrumbs.RelatedEntitiesName = string.Format(MobileUtility.GetResource("RelatedEntitiesText"), HttpUtility.HtmlEncode(_entityForm.CurrentEntityMetadata.DisplayCollectionName));
			breadCrumbs.RelatedEntitiesUrl = "RelatedEntities.aspx?relatedEntity="
											+ _entityForm.CurrentEntityMetadata.Name
											+ "&entity="
											+ referencingMetadata.Name
											+ "&"
											+ referencingMetadata.PrimaryKey
											+ "="
											+ referencingId
											+ "&instancename="
											+ referencingRecordName;
											
			if (_entityForm.RecordName != null && _entityForm.RecordName.Length != 0)
			{
				breadCrumbs.RelatedEntityPageName = HttpUtility.HtmlEncode(_entityForm.RecordName);
			}
			else
			{
				throw new ArgumentNullException(MobileUtility.GetResource("NameFieldIsRequired"));
			}

			if (_entityForm.RecordId != Guid.Empty)
			{
				breadCrumbs.RelatedEntityPageUrl = "EntityReadOnly.aspx?entity="
													+ _entityForm.CurrentEntityMetadata.Name
													+ "&"
													+ _entityForm.CurrentEntityMetadata.PrimaryKey
													+ "="
													+ _entityForm.RecordId.ToString("b")
													+ "&referencingentity="
													+ referencingMetadata.Name
													+ "&referencingrecordname="
													+ referencingRecordName
													+ "&referencingid="													
													+ referencingGuid.ToString("b");
			}
			else
			{
				throw new ArgumentNullException(MobileUtility.GetResource("RecordIdIsRequired"));
			}		
		}
		else
		{
			breadCrumbs.EntityHomePageName = HttpUtility.HtmlEncode(_entityForm.CurrentEntityMetadata.DisplayCollectionName);
			breadCrumbs.EntityHomePageUrl = "EntityHome.aspx?entity=" + _entityForm.CurrentEntityMetadata.Name;

			if (_entityForm.RecordName != null && _entityForm.RecordName.Length != 0)
			{
				breadCrumbs.RecordPageName = HttpUtility.HtmlEncode(_entityForm.RecordName);
			}
			else
			{
				breadCrumbs.RecordPageName = string.Format(MobileUtility.GetResource("NewRecordText"), _entityForm.CurrentEntityMetadata.DisplayName);
			}

			breadCrumbs.RecordPageUrl = "EntityEdit.aspx?entity=" + _entityForm.CurrentEntityMetadata.Name;

			if (_entityForm.RecordId != Guid.Empty)
			{
				breadCrumbs.RecordPageUrl += "&" + _entityForm.CurrentEntityMetadata.PrimaryKey + "=" + _entityForm.RecordId.ToString("b");
			}			
		}

		Master.Page.Title = this.GetDefaultPageTitle() + " - " + HttpUtility.HtmlEncode(_entityForm.RecordName);
	}
}
