// =======================================================================
//  File: EntityHome.aspx.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Globalization;
using System.Xml;
using System.Text;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.Helpers;
using Microsoft.Crm.Sandbox.Mobile.Helpers.Collection;
/// <summary>
/// 
/// </summary>
public partial class EntityHome : Mobile
{
	protected Guid _defaultQueryID;
	protected EntityMetadata _eMeta;
	private const string ENTITY_REQUEST_INDEX = "entity";
	private const int HOME_VIEW = 0;
	public string _masterPagePrefix;

	protected void Page_Load(object sender, EventArgs e)
    {		
		// Get the entity meta data
		this._eMeta = MetadataCache.GetEntityMetadata(Request.QueryString[ENTITY_REQUEST_INDEX].ToLower());

		BreadCrumbsControl breadCrumbs = Master.FindControl("BreadCrumbs") as BreadCrumbsControl;

		breadCrumbs.HomePageName = MobileUtility.GetResource("HomePageText");
		breadCrumbs.HomePageUrl = "Home.aspx";

		breadCrumbs.EntityHomePageName = HttpUtility.HtmlEncode(_eMeta.DisplayCollectionName);
        breadCrumbs.EntityHomePageUrl = HttpUtility.HtmlEncode("EntityHome.aspx?entity=" + Request.QueryString[ENTITY_REQUEST_INDEX]);

       	//Check to see if this is the first time accessing the page
		if (!Page.IsPostBack)
		{
			//If this is the first time accessing the page, then populate the view dropdown list
			if (_eMeta != null)
			{
				this.PopulateViewListBox();
			}
		}
		else
		{
			//If this is not the first time accessing the page then the query id becomes the value in the view dropdownlist
			this._defaultQueryID = new Guid(this.ViewsListBox.SelectedValue);
		}
		
		//Set the title of the page
		Master.Page.Title = this.GetDefaultPageTitle() + " - " + HttpUtility.HtmlEncode(this._eMeta.DisplayCollectionName);
		ViewLabel.InnerText = MobileUtility.GetResource("ViewLabelText") + ":";
		LookForLabel.InnerText = MobileUtility.GetResource("LookForLabelText") + ":";
		GoButton.Text = MobileUtility.GetResource("GoButtonText");
		FindButton.Text = MobileUtility.GetResource("FindButtonText");		

		//See if the entity is one that can be edited
		if(!MobileUtility.IsEntityReadOnly(_eMeta.Name))
		{
			//Set the create buttons properties
			if (MobileUtility.DoesUserHavePrivilege(_eMeta.Name, PrivilegeType.Create) && MobileUtility.DoesUserHavePrivilege(_eMeta.Name, PrivilegeType.Write))
			{
				StringBuilder html = new StringBuilder();
				html.AppendFormat("<b class=\"label\"><a id=\"createButton\" href=\"EntityEdit.aspx?entity={0}\">", _eMeta.Name);
				html.AppendFormat(MobileUtility.GetResource("NewRecordText"), HttpUtility.HtmlEncode(_eMeta.DisplayName));
				html.AppendFormat("</a></b>"); 
			
				CreateButtonHolder.InnerHtml = html.ToString();
			}
		}
			
		ViewsListBox.Width = Unit.Percentage(100);
		GoButton.Width = Unit.Percentage(80);
		entitySearchTerm.Width = Unit.Percentage(100);
		FindButton.Width = Unit.Percentage(80);
    }

	/// <summary>
	/// This method will run a saved query on the instances of an entity based on the the query id
	/// from the views dropdown list. After the saved query is executed, then a list of links for the instances
	/// of the entity are printed. The value that is displayed is the value of the primary field attribute. 
	/// 
	/// </summary>
	public void ExecuteAndPrintSavedQuery()
	{
		//Create the saved query execution object and set it's id
		ExecuteByIdSavedQueryRequest savedQueryExec = new ExecuteByIdSavedQueryRequest();
		savedQueryExec.EntityId = this._defaultQueryID;

		//Execute the saved query
		ExecuteByIdSavedQueryResponse savedQueryResponse = (ExecuteByIdSavedQueryResponse)this.CurrentCrmService.Execute(savedQueryExec);

		//Create an xml document based on the results of the query
		XmlDocument xmlDoc = new XmlDocument();
		xmlDoc.LoadXml(savedQueryResponse.String);

		//Get all the result nodes
		XmlNodeList instanceNodes = xmlDoc.GetElementsByTagName("result");

		//Print out the links of the instances
		Response.Write("<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\"><tr><td class=\"outerList\"><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\"><tr><td valign=\"top\" class=\"innerList\">");

		foreach (XmlNode node in instanceNodes)
		{
			XmlNode primaryFieldNode = node.SelectSingleNode(_eMeta.PrimaryField);
			XmlNode primaryKeyNode = node.SelectSingleNode(_eMeta.PrimaryKey);

			if (primaryFieldNode != null && primaryKeyNode != null)
			{
				Response.Write("<a href=\"EntityReadOnly.aspx?entity=");
				Response.Write(_eMeta.Name);
				Response.Write("&");
				Response.Write(_eMeta.PrimaryKey);
				Response.Write("=");
				Response.Write(primaryKeyNode.InnerText);
				Response.Write("\">");
				Response.Write(HttpUtility.HtmlEncode(primaryFieldNode.InnerText));
				Response.Write("</a><br>");
			}
		}

		if (instanceNodes.Count == 0)
		{
			Response.Write("<div class=\"noRecordsFound\">");
			Response.Write(string.Format(MobileUtility.GetResource("NoRecordsFoundText"), HttpUtility.HtmlEncode(_eMeta.DisplayName)));
			Response.Write("</div>");
		}

		Response.Write("</td></tr></table></td></tr></table>");
	}

	/// <summary>
	/// This method will take the search term in the form and run a search
	/// with a "like (begins with)" search against the primary attribute field
	/// of the instances. All matches will be printed as links with the primary attribute
	/// field value.
	/// </summary>
	public void SearchAndPrintInstances()
	{		
		// Create the Request Object
		RetrieveMultipleRequest retrieve = new RetrieveMultipleRequest();
		retrieve.ReturnDynamicEntities = true;

		//Create the query helper for setting the query
		QueryExpressionHelper helper = new QueryExpressionHelper(_eMeta.Name);
		helper.Columns.AddColumns(_eMeta.PrimaryField, _eMeta.PrimaryKey);
		helper.Criteria.Conditions.AddCondition(_eMeta.PrimaryField, ConditionOperator.Like, Request.Form[_masterPagePrefix + entitySearchTerm.ID] + "%");

		//Get the maximum number of returned instances specified in the web.config
		helper.PageInfo.Count = this.GetMaxInstancesReturned();
		helper.PageInfo.PageNumber = 1;

		//Set the query
		retrieve.Query = helper.Query;

		// Execute the Request
		RetrieveMultipleResponse retrieved = (RetrieveMultipleResponse)this.CurrentCrmService.Execute(retrieve);

		//Print out the instances links with the primary attribute field value
		Response.Write("<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\"><tr><td class=\"outerList\"><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\"><tr><td valign=\"top\" class=\"innerList\">");

		for (int i = 0; i < retrieved.BusinessEntityCollection.BusinessEntities.Length; i++)
		{
			DynamicEntity entityRetrieved = retrieved.BusinessEntityCollection.BusinessEntities[i] as DynamicEntity;

			Response.Write("<a href=\"EntityReadOnly.aspx?entity=");
			Response.Write(_eMeta.Name);
			Response.Write("&");
			Response.Write(_eMeta.PrimaryKey);
			Response.Write("=");
			Response.Write(MobileUtility.GetDynamicEntityPropertyValue(entityRetrieved, _eMeta.PrimaryKey));
			Response.Write("\">");
			Response.Write(HttpUtility.HtmlEncode(MobileUtility.GetDynamicEntityPropertyValue(entityRetrieved, _eMeta.PrimaryField)));
			Response.Write("</a><br>");
		}

		if (retrieved.BusinessEntityCollection.BusinessEntities.Length == 0)
		{
			Response.Write("<div class=\"noRecordsFound\">");
			Response.Write(string.Format(MobileUtility.GetResource("NoRecordsFoundText"), HttpUtility.HtmlEncode(_eMeta.DisplayName)));
			Response.Write("</div>");
		}

		Response.Write("</td></tr></table></td></tr></table>");
	}

	/// <summary>
	/// This method will search for system and user views of the entity
	/// and then populate the dropdown list. The list is used for querying
	/// instances based on the custom saved queries that the views represent.
	/// </summary>
	private void PopulateViewListBox()
	{
		// Retrieves all system views that have fetchxml
		QueryExpressionHelper helper = new QueryExpressionHelper(EntityName.savedquery);
		helper.Columns.AddColumns("savedqueryid", "name", "isdefault");
		helper.Orders.Add("isdefault", OrderType.Descending);
		helper.Orders.Add("name", OrderType.Ascending);
		helper.Criteria.Conditions.AddCondition("querytype", ConditionOperator.Equal, HOME_VIEW);
		helper.Criteria.Conditions.AddCondition("returnedtypecode", ConditionOperator.Equal, Int32.Parse(_eMeta.ObjectTypeCode.ToString(), CultureInfo.InvariantCulture));
		helper.Criteria.Conditions.AddCondition("fetchxml", ConditionOperator.NotNull);

		// Execute the Request
		BusinessEntityCollection retrieved = this.CurrentCrmService.RetrieveMultiple(helper.Query);

		// Iterate through all entities and add their attributes and relationships to the dropdown list
		for (int i = 0; i < retrieved.BusinessEntities.Length; i++)
		{
			savedquery sq = retrieved.BusinessEntities[i] as savedquery;
			ViewsListBox.Items.Add(new ListItem(HttpUtility.HtmlEncode(sq.name), sq.savedqueryid.Value.ToString("b")));

			//If the saved query is the default, then make it the default id
			if (sq.isdefault.Value)
			{
				this._defaultQueryID = sq.savedqueryid.Value;
			}
		}

		//Check to see if th default query id was set.
		//Sometimes there may be queries that do not have
		//the isDefault set to 1. In that case, the default query id
		//would not be set. So we will set it to the first view in the
		//dropdown list.
		if (this._defaultQueryID == Guid.Empty && ViewsListBox.Items.Count > 0)
		{
			this._defaultQueryID = new Guid(ViewsListBox.Items[0].Value);
		}
	}
}
