// =======================================================================
//  File: EntityReadOnly.aspx.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections;
using System.Xml;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;

/// <summary>
/// This page is responsible for displaying the readonly form
/// of the instance.
/// </summary>
public partial class EntityReadOnly : Mobile
{
	protected EntityForm _entityForm;

	protected void Page_Load(object sender, EventArgs e)
	{
		//Create the entity form for the instance
		_entityForm = new EntityForm(this, true);

		//Add the entity form	
		TheForm.Controls.Add(_entityForm);

		BreadCrumbsControl breadCrumbs = Master.FindControl("BreadCrumbs") as BreadCrumbsControl;

		breadCrumbs.HomePageName = MobileUtility.GetResource("HomePageText");
		breadCrumbs.HomePageUrl = "Home.aspx";
		
		string referencingEntity = HttpUtility.HtmlEncode(Request.QueryString["referencingentity"]);
		string referencingRecordName = HttpUtility.HtmlEncode(Request.QueryString["referencingrecordname"]);
		string referencingId = HttpUtility.HtmlEncode(Request.QueryString["referencingid"]);		

		// Coming from the related entities home page will give us the above querystring values
		// They must all be present in order for this page to "act" as the 5th layer of the BC control
		if ((referencingEntity != null && referencingEntity.Length != 0) &&
			(referencingRecordName != null && referencingRecordName.Length != 0) &&
			(referencingId != null && referencingId.Length != 0))
		{
			Guid referencingGuid = new Guid(referencingId);
			
			EntityMetadata referencingMetadata = MetadataCache.GetEntityMetadata(referencingEntity);
			breadCrumbs.EntityHomePageName = HttpUtility.HtmlEncode(referencingMetadata.DisplayCollectionName);
			breadCrumbs.EntityHomePageUrl = "EntityHome.aspx?entity=" + referencingMetadata.Name;

			breadCrumbs.RecordPageName = referencingRecordName;				
			breadCrumbs.RecordPageUrl = "EntityReadOnly.aspx?entity=" + referencingMetadata.Name + "&" + referencingMetadata.PrimaryKey + "=" + referencingId;
			
			breadCrumbs.RelatedEntitiesName = string.Format(MobileUtility.GetResource("RelatedEntitiesText"), HttpUtility.HtmlEncode(_entityForm.CurrentEntityMetadata.DisplayCollectionName));
			breadCrumbs.RelatedEntitiesUrl = "RelatedEntities.aspx?relatedEntity="
											+ _entityForm.CurrentEntityMetadata.Name
											+ "&entity="
											+ referencingMetadata.Name
											+ "&"
											+ referencingMetadata.PrimaryKey
											+ "="
											+ referencingGuid.ToString("b")
											+ "&instancename="
											+ referencingRecordName;
											
			// Related entities are already created and should always have names and ids								
			if (_entityForm.RecordName != null && _entityForm.RecordName.Length != 0)
			{
				breadCrumbs.RelatedEntityPageName = HttpUtility.HtmlEncode(_entityForm.RecordName);
			}
			else
			{
				throw new ArgumentNullException(MobileUtility.GetResource("NameFieldIsRequired"));
			}

			if (_entityForm.RecordId != Guid.Empty)
			{
				breadCrumbs.RelatedEntityPageUrl = "EntityReadOnly.aspx?entity="
													+ _entityForm.CurrentEntityMetadata.Name
													+ "&"
													+ _entityForm.CurrentEntityMetadata.PrimaryKey
													+ "="
													+ _entityForm.RecordId.ToString("b")
													+ "&referencingentity="
													+ referencingMetadata.Name
													+ "&referencingrecordname="
													+ referencingRecordName
													+ "&referencingid="
													+ referencingGuid.ToString("b");
			}
			else
			{
				throw new ArgumentNullException(MobileUtility.GetResource("RecordIdIsRequired"));
			}		 
		}
		else
		{
			breadCrumbs.EntityHomePageName = HttpUtility.HtmlEncode(_entityForm.CurrentEntityMetadata.DisplayCollectionName);
			breadCrumbs.EntityHomePageUrl = "EntityHome.aspx?entity=" + _entityForm.CurrentEntityMetadata.Name;

			if (_entityForm.RecordName != null && _entityForm.RecordName.Length != 0)
			{
				breadCrumbs.RecordPageName = HttpUtility.HtmlEncode(_entityForm.RecordName);
			}
			else
			{
				throw new ArgumentNullException(MobileUtility.GetResource("NameFieldIsRequired"));
			}

			if (_entityForm.RecordId != Guid.Empty)
			{				
				breadCrumbs.RecordPageUrl = "EntityReadOnly.aspx?entity=" + _entityForm.CurrentEntityMetadata.Name + "&" + _entityForm.CurrentEntityMetadata.PrimaryKey + "=" + _entityForm.RecordId.ToString("b");
			}
			else
			{
				throw new ArgumentNullException(MobileUtility.GetResource("RecordIdIsRequired"));
			}		
			
			// TODO: WE ARE STOPPING RELATED ENTITIES AT THE RELATED ENTITY FOR NOW
			// The BC control does not support an infinite number of links at the moment, so we are allowing
			// 5 levels of depth.			

			//The array list that will hold the related entity names
			//That way we don't have more than one entity of the same name as a link
			ArrayList relatedEntitiesList = new ArrayList();

			//Get the relationships that relate to the current entity instance
			RelationshipMetadata[] relationships = this._entityForm.CurrentEntityMetadata.ReferencesTo;

			//Add the related entity names to the list if the entity exists in the config xml
			foreach (RelationshipMetadata relationship in relationships)
			{
				XmlNode entityNode = MobileUtility.GetEntityNode(relationship.ReferencingEntity);

				if (entityNode != null)
				{
					if (!relatedEntitiesList.Contains(relationship.ReferencingEntity))
					{
						relatedEntitiesList.Add(relationship.ReferencingEntity);
					}
				}
			}

            // Add HR before the Related Entity area
            FormBar formBar = new FormBar();
            Form.Controls.Add(formBar);

			//The label for the related entities section
			BoldLabel relatedEntitiesLabel = new BoldLabel();
			relatedEntitiesLabel.Text = MobileUtility.GetResource("RelatedEntitiesListText") + ":";

			Form.Controls.Add(relatedEntitiesLabel);

			//We must reorder the list of related entities to the order in
			//the config xml
			
			//Get the entities in the config xml
			XmlNodeList entitiesList = this.MobileConfig.SelectNodes("MobileClient/Entities/Entity");
			
			//Temporary ArrayList for the reordering
			ArrayList tmpEntities = new ArrayList();
			
			//Cycle through the entities and if they exist in the related
			//entities array list, then add them to the temp list.
			foreach(XmlNode lEntity in entitiesList)
			{
				string lEntityName = lEntity.Attributes["name"].Value;
				
				if(relatedEntitiesList.Contains(lEntityName))
				{
					tmpEntities.Add(lEntityName);
				}
			}
			
			//relatedEntities list is now the new ordered list
			relatedEntitiesList = tmpEntities;

			//The table that holds rows of links to the related entities
			HtmlTable relatedEntitiesTable = new HtmlTable();

			foreach (Object entityName in relatedEntitiesList)
			{
				//See if the user has read privilege to the entity
				//If they do, then proceed to add the entity link
				if (MobileUtility.DoesUserHavePrivilege(entityName.ToString(), PrivilegeType.Read))
				{
					//Create a new row and cell to hold the entity link
					HtmlTableRow row = new HtmlTableRow();
					HtmlTableCell cell = new HtmlTableCell();

					//Create the link for the related entity
					HyperLink entityLink = new HyperLink();

					//Retrieve the entity metadata for the appropriate display name of the link
					EntityMetadata targetEntityMeta = MetadataCache.GetEntityMetadata(entityName.ToString());

					entityLink.Text = HttpUtility.HtmlEncode(targetEntityMeta.DisplayCollectionName);

					//Set the navigation url using the related entity name and the metadata for the current instance
					entityLink.NavigateUrl = "RelatedEntities.aspx?relatedEntity="
											+ entityName.ToString()
											+ "&entity="
											+ this._entityForm.CurrentEntityMetadata.Name
											+ "&"
											+ this._entityForm.CurrentEntityMetadata.PrimaryKey
											+ "="
											+ this._entityForm.RecordId.ToString()
											+ "&instancename="
											+ HttpUtility.HtmlEncode(this._entityForm.RecordName);

					//Add the link to the new cell
					cell.Controls.Add(entityLink);

					//Add the cell and row to the table
					row.Controls.Add(cell);
					relatedEntitiesTable.Controls.Add(row);
				}
			}

			//Add the related entity links table to the page
			Form.Controls.Add(relatedEntitiesTable);
		}

		Master.Page.Title = this.GetDefaultPageTitle() + " - " + HttpUtility.HtmlEncode(_entityForm.RecordName);	
	}
}
