// =======================================================================
//  File: Home.aspx.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;

/// <summary>
/// This class represents the home page of the mobile client application
/// </summary>
public partial class Home : Mobile
{
    protected void Page_Load(object sender, EventArgs e)
    {		
        Master.Page.Title = this.GetDefaultPageTitle();
       
       	BreadCrumbsControl breadCrumbs = Master.FindControl("BreadCrumbs") as BreadCrumbsControl;

		breadCrumbs.HomePageName = MobileUtility.GetResource("HomePageText");
		breadCrumbs.HomePageUrl = "Home.aspx";
    }
    
    /// <summary>
    /// This method will get all the entity nodes of the configuration xml
	/// and print out links to the entity home page of the respective entity
    /// </summary>
    public void PrintListOfEntities()
    {		
		//Get all the entity nodes
		XmlNodeList entityElements = this.MobileConfig.GetElementsByTagName("Entity");

		//Write out the table with the entity links to EntityHome
		Response.Write("<table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\"><tr><td class=\"outerList\"><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\"><tr><td valign=\"top\" class=\"innerList\">");

		foreach (XmlNode entity in entityElements)
		{
			string name = entity.Attributes.GetNamedItem("name").Value;			
			
			if (name != EntityName.annotation.ToString() &&
				name != EntityName.customeraddress.ToString())
			{
				if (MobileUtility.DoesUserHavePrivilege(name, PrivilegeType.Read))
				{
					Response.Write("<a href=\"EntityHome.aspx?entity=");
					Response.Write(name);
					Response.Write("\">");

					EntityMetadata entityInfo = MetadataCache.GetEntityMetadata(name);

					Response.Write(HttpUtility.HtmlEncode(entityInfo.DisplayCollectionName));
					Response.Write("</a><br>");
				}
			}
		}		

		if (entityElements.Count == 0)
		{
			Response.Write("<div class=\"noRecordsFound\">");
			Response.Write(MobileUtility.GetResource("NoEntitiesFoundText"));
			Response.Write("</div>");
		}

		Response.Write("</td></tr></table></td></tr></table>");
    }	
}