// =======================================================================
//  File: RelatedEntities.aspx.cs
// =======================================================================
//
//  This file is part of the Microsoft Dynamics CRM 3.0 Thin Mobile Client
//  Code Samples.
//
//  Copyright (C) Microsoft Corporation.  All rights reserved.
//
//  This source code is intended only as a supplement to Microsoft
//  Development Tools and/or on-line documentation.  See these other
//  materials for detailed information regarding Microsoft code samples.
//
//  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//  PARTICULAR PURPOSE.
//
// =======================================================================

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Microsoft.Crm.Sandbox.Mobile.CrmServiceSdk;
using Microsoft.Crm.Sandbox.Mobile.MetadataServiceSdk;

using Microsoft.Crm.Sandbox.Mobile.Helpers;
using Microsoft.Crm.Sandbox.Mobile.Helpers.Collection;

public partial class RelatedEntities : Mobile
{
    protected void Page_Load(object sender, EventArgs e)
    {		
		//Get the name of the referenced entity and the entity that is related
		string relatedEntity = Request.QueryString["relatedEntity"];
		string entityName = Request.QueryString["entity"];

		//Retrieve the entity meta data for both the referenced entity and the related entity
		EntityMetadata eMeta = MetadataCache.GetEntityMetadata(entityName);
		EntityMetadata targetEntityMeta = MetadataCache.GetEntityMetadata(relatedEntity);

		Master.Page.Title = this.GetDefaultPageTitle() + " - " + HttpUtility.HtmlEncode(Request.QueryString["instancename"]) + " - " + string.Format(MobileUtility.GetResource("RelatedEntitiesText"), HttpUtility.HtmlEncode(targetEntityMeta.DisplayCollectionName));
		
		//Get the guid value of the referenced entity instance
		string entityId = Request.QueryString[eMeta.PrimaryKey];

		BreadCrumbsControl breadCrumbs = Master.FindControl("BreadCrumbs") as BreadCrumbsControl;

		//Set the labels and links of the breadcrumbs navigation control
		breadCrumbs.HomePageName = MobileUtility.GetResource("HomePageText");
		breadCrumbs.HomePageUrl = "Home.aspx";

		breadCrumbs.EntityHomePageName = HttpUtility.HtmlEncode(eMeta.DisplayCollectionName);
		breadCrumbs.EntityHomePageUrl = "EntityHome.aspx?entity=" + entityName;

		breadCrumbs.RecordPageName = HttpUtility.HtmlEncode(Request.QueryString["instancename"]);
		breadCrumbs.RecordPageUrl = "EntityReadOnly.aspx?entity=" 
										+ eMeta.Name 
										+ "&" 
										+ eMeta.PrimaryKey 
										+ "=" 
										+ entityId;

		breadCrumbs.RelatedEntitiesName = string.Format(MobileUtility.GetResource("RelatedEntitiesText"), HttpUtility.HtmlEncode(targetEntityMeta.DisplayCollectionName));
		breadCrumbs.RelatedEntitiesUrl = "RelatedEntities.aspx?relatedEntity=" 
										+ relatedEntity 
										+ "&entity=" 
										+ entityName 
										+ "&" 
										+ eMeta.PrimaryKey 
										+ "=" 
										+ entityId
										+ "&instancename="
										+ HttpUtility.HtmlEncode(Request.QueryString["instancename"]);										
	
		//Add the header of the related entity instances to the page
		BoldLabel rLabel = new BoldLabel();
		rLabel.Text = string.Format(MobileUtility.GetResource("RelatedEntitiesText"), HttpUtility.HtmlEncode(targetEntityMeta.DisplayCollectionName)) + ":";
		Form.Controls.Add(rLabel);
				
		if(eMeta != null)
		{	
			if(entityId != null)
			{
				//Create the table to hold the links of the related entity instances
				HtmlTable entityTable = new HtmlTable();
			
				//Get the relationship meta data of the entity instance referenced
				RelationshipMetadata[] relationships = eMeta.ReferencesTo;
				
				//List to store the guid of each fetched related instance
				ArrayList guidList = new ArrayList();
				
				foreach(RelationshipMetadata relationship in relationships)
				{
					if(String.Compare(relationship.ReferencingEntity,relatedEntity,false) == 0)
					{
						try
						{
							// Create the Request Object
							RetrieveMultipleRequest retrieve = new RetrieveMultipleRequest();
							retrieve.ReturnDynamicEntities = true;

							//Create the query helper for setting the query
							QueryExpressionHelper helper = new QueryExpressionHelper(targetEntityMeta.Name);
							helper.Columns.AddColumns(targetEntityMeta.PrimaryField,targetEntityMeta.PrimaryKey);
							helper.Criteria.Conditions.AddCondition(relationship.ReferencingAttribute, ConditionOperator.Equal, entityId);

							//Add an additional condition if the related entity is 
							//a customer address. This way we can get the additional addresses
							//that are not managed by crm
							if(relatedEntity.Equals("customeraddress"))
							{
								helper.Criteria.Conditions.AddCondition("addressnumber",ConditionOperator.GreaterThan,2);
							}

							//Get the maximum number of returned instances specified in the web.config
							helper.PageInfo.Count = this.GetMaxInstancesReturned();
							helper.PageInfo.PageNumber = 1;

							//Set the query
							retrieve.Query = helper.Query;

							// Execute the Request
							RetrieveMultipleResponse retrieved = (RetrieveMultipleResponse)this.CurrentCrmService.Execute(retrieve);
							BusinessEntity[] bEntities = retrieved.BusinessEntityCollection.BusinessEntities;
							
							for(int i = 0;i < bEntities.Length;i++)
							{
								DynamicEntity de = (DynamicEntity)bEntities[i];
								
								//Get the value of the guid of the fetched record
								string recordIdValue = MobileUtility.GetDynamicEntityPropertyValue(de,targetEntityMeta.PrimaryKey);
								
								//If the guid of this fetched related instance does not exist
								//in the guid list then proceed to security checks and link creation
								if(!guidList.Contains(recordIdValue))
								{
									//Add the guid value to the guid list
									guidList.Add(recordIdValue);
									
									bool passedSecurity = false;
									
									//See if the user has read rights to the record or
									//read privilege to the entity. The type of entity determines
									//the security check
									if(!relatedEntity.Equals("customeraddress"))
									{
										//Check to see if the user has read privilege to the entity
										passedSecurity = MobileUtility.DoesUserHaveRightsToRecord(relatedEntity,new Guid(recordIdValue),AccessRights.ReadAccess);
									}
									else
									{
										//Check to see if the user has read rights to the record
										passedSecurity = MobileUtility.DoesUserHavePrivilege(relatedEntity,PrivilegeType.Read);
									}
									
									if(passedSecurity)
									{
										//Create the row and cell that will hold the link of the related instance
										HtmlTableRow row = new HtmlTableRow();
										HtmlTableCell cell = new HtmlTableCell();
									
										//Create the link of the related instance
										HyperLink entityLink = new HyperLink();
									
										//Make the text of the link the value of the primary field of the retrieved entity
										entityLink.Text = HttpUtility.HtmlEncode(MobileUtility.GetDynamicEntityPropertyValue(de,targetEntityMeta.PrimaryField));
									
										//Set the navigation url using the key value of the entity and the meta data of the
										//related entity
										entityLink.NavigateUrl = "EntityReadOnly.aspx?entity="
																+ targetEntityMeta.Name
																+ "&"
																+ targetEntityMeta.PrimaryKey
																+ "="
																+ recordIdValue
																+ "&referencingentity="
																+ entityName
																+ "&referencingrecordname="
																+ HttpUtility.HtmlEncode(Request.QueryString["instancename"])
																+ "&referencingid="															
																+ entityId;
									
										//Add the row to the entity instance table						
										cell.Controls.Add(entityLink);
										row.Controls.Add(cell);
										entityTable.Controls.Add(row);
									}
								}
							}
						}
						catch(Exception except)
						{
							Response.Write(except.StackTrace);
						}
					}
				}
				
				//Add the entity instance table to the page
				Form.Controls.Add(entityTable);
			}
		}
    }
}
